# Benchmarking Suite
# Copyright 2014-2017 Engineering Ingegneria Informatica S.p.A.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Developed in the ARTIST EU project (www.artist-project.eu) and in the
# CloudPerfect EU project (https://cloudperfect.eu/)
import argparse
import logging
import signal
import sys
import json

import os
from flask import Flask
from flask_restplus import Swagger

from benchsuite.rest.apiv1 import blueprint as blueprint1
from benchsuite.rest.apiv1 import api as apiv1
app = Flask(__name__)


#app.config.SWAGGER_UI_JSONEDITOR = True
app.config.SWAGGER_UI_DOC_EXPANSION = 'list'

app.register_blueprint(blueprint1)

def on_exit(sig, func=None):
    print('Bye bye...')
    sys.exit(1)


def dump_swagger_specs():
    app.config['SERVER_NAME'] = 'example.org:80'
    with app.app_context():
        print(json.dumps(Swagger(apiv1).as_dict(), indent=2))


if __name__ == '__main__':

    signal.signal(signal.SIGINT, on_exit)

    logging.basicConfig(
        level=logging.DEBUG,
        stream=sys.stdout)

    parser = argparse.ArgumentParser(prog='benchsuite-rest')
    parser.add_argument('--dump-specs', action='store_true',
                        help='dumps the Swagger specification')
    parser.add_argument('--listen', '-l', type=str,
                        help='set the listening host and port (e.g. 0.0.0.0:80). '
                             'If not specified, the default is 127.0.0.1:5000')


    # parse the arguments. They are taken both from the command line and
    # the BENCHSUITE_REST_OPTS environment variable
    env_opts = os.getenv('BENCHSUITE_REST_OPTS', '').split()
    args = parser.parse_args(sys.argv[1:] + env_opts)



    if args.dump_specs:
        dump_swagger_specs()
        sys.exit(0)



    host = '127.0.0.1'
    port = 5000

    if args.listen:
        if ':' in args.listen:
            t = args.listen.rsplit(':', 1)
            host = t[0]
            port = int(t[1])
        else:
            print('ERROR: wrong format for listening address. '
                  'Must be in the format <host>:<port>')
            sys.exit(1)

    #TODO: use nginx here instead of the internal server
    print('Using internal server. Not use this in production!!!')
    app.run(host=host, port=port, debug=True)




# Benchmarking Suite
# Copyright 2014-2017 Engineering Ingegneria Informatica S.p.A.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Developed in the ARTIST EU project (www.artist-project.eu) and in the
# CloudPerfect EU project (https://cloudperfect.eu/)

from datetime import datetime

from benchsuite.core.controller import BenchmarkingController
from flask_restplus import Namespace, Resource, fields
from benchsuite.rest.apiv1.model import timestamp_to_string

api = Namespace('executions', description='Executions operations')


# register models from the other namespaces
benchmark_model = api.model('Benchmark', {
    'tool_id': fields.String,
    'workload_id': fields.String,
    'tool_name': fields.String,
    'workload_name': fields.String,
    'workload_description': fields.String,
})

vm_model = api.model('VM', {
    'id': fields.String(
        description='the id of the VM as assigned by the Cloud Management Software (e.g. OpenStack)'),

    'ip': fields.String(
        description='the public ip of the VM'),

    'platform': fields.String(
        description='the VM platform (e.g. "Ubuntu")')
})

execution_env_model = api.model('ExecutionEnvironment', {
    'vms': fields.List(fields.Nested(vm_model))
})

execution_model = api.model('Execution', {
    'id': fields.String,
    'test': fields.Nested(benchmark_model),
    'exec_env': fields.Nested(execution_env_model),
    'created': fields.String(attribute=lambda x: timestamp_to_string(x.created))
})

new_execution_model = api.model('NewExecution', {
    'tool': fields.String,
    'workload': fields.String
})

execution_command_info_model = api.model('ExecutionCommand', {
    'started': fields.String(attribute=lambda x: timestamp_to_string(x.started)),
    'duration': fields.String
})


@api.route('/')
class ExecutionList(Resource):

    @api.marshal_with(execution_model)
    def get(self):
        with BenchmarkingController() as bc:
            return list(bc.list_executions())


@api.route('/<string:exec_id>')
class Execution(Resource):

    @api.marshal_with(execution_model)
    def get(self, exec_id):
        with BenchmarkingController() as bc:
            return bc.get_execution(exec_id)


@api.route('/<string:exec_id>/prepare')
class ExecutionPrepareACtion(Resource):

    @api.marshal_with(execution_command_info_model, code=200, description='Runs the prepare commands')
    def post(self, exec_id):
        with BenchmarkingController() as bc:
            return bc.prepare_execution(exec_id)



@api.route('/<string:exec_id>/run')
class ExecutionRunACtion(Resource):

    @api.marshal_with(execution_command_info_model, code=200, description='Runs the run commands')
    def post(self, exec_id):
        with BenchmarkingController() as bc:
            return bc.run_execution(exec_id)



# Benchmarking Suite
# Copyright 2014-2017 Engineering Ingegneria Informatica S.p.A.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Developed in the ARTIST EU project (www.artist-project.eu) and in the
# CloudPerfect EU project (https://cloudperfect.eu/)

# 'benchsuite' is not a module, but a namespace.
# see: https://packaging.python.org/guides/packaging-namespace-packages/#
__import__('pkg_resources').declare_namespace(__name__)


# Benchmarking Suite
# Copyright 2014-2017 Engineering Ingegneria Informatica S.p.A.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Developed in the ARTIST EU project (www.artist-project.eu) and in the
# CloudPerfect EU project (https://cloudperfect.eu/)

from benchsuite.core.controller import BenchmarkingController
from flask_restplus import Namespace, Resource, fields
from datetime import datetime
from benchsuite.rest.apiv1.executions import new_execution_model, execution_model
from benchsuite.rest.apiv1.model import APIException

api = Namespace('sessions', description='Benchmarking Sessions operations')

api.models[execution_model.name] = execution_model
api.models[new_execution_model.name] = new_execution_model

provider_model = api.model('Provider', {
    'id': fields.String,
    'type': fields.String
})

session_model = api.model('Session', {
    'id': fields.String,
    'provider': fields.Nested(provider_model),
    'created': fields.String(attribute=lambda x: datetime.fromtimestamp(x.created).strftime('%Y-%m-%d %H:%M:%S')),
    'executions': fields.Nested(execution_model, as_list=True, attribute=lambda x: list(x.executions.values()))
})

new_session_model = api.model('NewSession', {
    'provider': fields.String(required=False),
    'service': fields.String(required=False),
    'config': fields.String(required=False, description="Specify the provider configuration as a string without "
                                                        "loading it from the Benchmarking Suite configuration folder")
})

@api.route('/')
class SessionList(Resource):

    @api.marshal_with(session_model, as_list=True, code=200, description='Returns the list of the existing sessions')
    def get(self):
        with BenchmarkingController() as bc:
            return list(bc.list_sessions())

    @api.expect(new_session_model)
    @api.marshal_with(session_model, code=204,
                      description='Creates a new benchmarking session. Returns the newly created session')
    def post(self):
        with BenchmarkingController() as bc:

            if 'provider' in self.api.payload and 'service' in self.api.payload:
                return bc.new_session(self.api.payload['provider'], self.api.payload['service'])

            if 'config' in self.api.payload:
                return bc.new_session_by_config(self.api.payload['config'])

            raise APIException('Either config or provider must be specified')


@api.route('/<string:session_id>')
@api.param('session_id', 'The id of the session')
class Session(Resource):

    @api.marshal_with(session_model, as_list=False, code=200, description='The session with the requested session id')
    def get(self, session_id):
        with BenchmarkingController() as bc:
            return bc.get_session(session_id)

    @api.response(204, description='Deletes a benchmarking session')
    def delete(self, session_id):
        with BenchmarkingController() as bc:
            bc.destroy_session(session_id)
        return '', 204


@api.route('/<string:session_id>/executions/')
class SessionExecution(Resource):

    @api.marshal_with(execution_model, code=200,
                      description='Returns the executions associated with this session')
    def get(self, session_id):
        with BenchmarkingController() as bc:
            return list(bc.get_session(session_id).list_executions())


    @api.expect(new_execution_model)
    @api.marshal_with(execution_model, code=200,
                      description='Creates a new execution in this benchmarking session. Returns the newly created execution')
    def post(self, session_id):
        with BenchmarkingController() as bc:
            return bc.new_execution(session_id, self.api.payload['tool'], self.api.payload['workload'])


# Benchmarking Suite
# Copyright 2014-2017 Engineering Ingegneria Informatica S.p.A.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Developed in the ARTIST EU project (www.artist-project.eu) and in the
# CloudPerfect EU project (https://cloudperfect.eu/)

from flask_restplus import Namespace, fields, Resource
from benchsuite.core.controller import BenchmarkingController


api = Namespace('benchmarks', description='Benchmarks operations')

#
# Models
#
benchmark_workload_config_model = api.model('BenchmarkWorkloadConfiguration', {
    'id': fields.String(
        description='The id of the workload (i.e. the name of the section containing the configuration)'),

    'workload_name': fields.String(
        description='the value of the "workload_name" field in the configuration'),

    'workload_description': fields.String(
        description='the value of the "workload_description" field in the configuration')
})

benchmark_config_model = api.model('BenchmarkConfiguration', {
    'id': fields.String(
        description='The id of the benchmark (i.e. the name of the file containing the configuration'),

    'tool_name': fields.String(
        description='the value of the "tool_name" field in the configuration'),

    'workloads': fields.List(fields.Nested(benchmark_workload_config_model))
})


#
# Routes
#

@api.route('/')
class BenchmarkConfigurationsList(Resource):

    @api.marshal_with(benchmark_config_model,
                      as_list=True,
                      code=200,
                      description='Returns the list of all benchmark tools configuration files found')
    def get(self):
        with BenchmarkingController() as bc:
            return bc.list_available_benchmark_cfgs()


@api.route('/<string:benchmark_id>')
class BenchmarkConfiguration(Resource):

    @api.marshal_with(benchmark_config_model,
                      code=200,
                      description="Returns one benchmark tool and all available workloads")
    def get(self, benchmark_id):
        with BenchmarkingController() as bc:
            return bc.get_benchmark_cfg(benchmark_id)


# Benchmarking Suite
# Copyright 2014-2017 Engineering Ingegneria Informatica S.p.A.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Developed in the ARTIST EU project (www.artist-project.eu) and in the
# CloudPerfect EU project (https://cloudperfect.eu/)


from flask import Blueprint
from flask_restplus import Api, fields, Resource

from benchsuite.core.model.exception import ControllerConfigurationException, BashCommandExecutionFailedException, \
    UndefinedSessionException, ProviderConfigurationException, BaseBenchmarkingSuiteException

blueprint = Blueprint('apiv1', __name__, url_prefix='/api/v1')

description = '''

A quick tutorial on how to use the API is available in the Benchmarking Suite online documentation at:


http://benchmarking-suite.readthedocs.io/en/latest/rest.html#quick-start

'''

api = Api(
    blueprint,
    title='Benchmarking Suite REST API',
    version='1.0',
    description=description,
    # All API metadatas
)

bash_command_failed_model = api.model('BashCommandExecutionFailed', {
    'message': fields.String,
    'stdout': fields.String(example='This will be the command stdout'),
    'stderr': fields.String(example='This will be the command stderr')
})



# default error handler
@api.errorhandler
def handle_custom_exception(error):
    return {'message': str(error)}, 500

@api.errorhandler(BashCommandExecutionFailedException)
def handle_command_failed_exception(error):
    return {'message': str(error), 'stdout': str(error.stdout), 'stderr': str(error.stderr)}, 400

@api.errorhandler(UndefinedSessionException)
def handle_undefined_session(error):
    return {'message': str(error)}, 400

from .executions import api as executions_ns
from .sessions import api as sessions_ns
from .providers import api as providers_ns
from .benchmarks import api as benchmarks_ns

api.add_namespace(sessions_ns)
api.add_namespace(executions_ns)
api.add_namespace(providers_ns)
api.add_namespace(benchmarks_ns)

